Attribute VB_Name = "Module1"
Option Explicit

Public Const InFile = 1
Public Const HtmlFile = 2
Public Const EventFile = 3
Public Const StyleFile = 4
Public Const VCalFile = 5
Public Const MaxEvents = 1000
Public Const MaxEventTypes = 50
Public Const MaxUnits = 25
Public Const TableName = "Calendar"

Public SelectedIDs(MaxEvents) As Long
Public Events(MaxEvents) As Cal
Public Sorted(MaxEvents) As SortCal
Public NumEvents As Integer
Public EventTypes(MaxEventTypes) As String
Public NumEventTypes As Integer
Public UnitNames(MaxUnits) As String
Public NumUnitNames As Integer

Public CalTitle As String
Public CalFileName As String
Public EventFileName As String
Public OutputDir As String
Public CalFilePrefix As String
Public VCalDir As String
Public VCalFileName As String

Public dbConnection As New Connection
Public DatabaseName As String

Public CalForm As String
Public CalDate As String

Public Type Cal
    EID As Long     ' unique ID of event
    EDate As Date   ' date of event
    ETime As String ' time of event (optional)
    EBegTime As Date    ' beginning time
    EEndTime As Date    ' ending time
    EDay As String  ' day of week
    EDesc As String ' description of event
    EStake As String ' Y if a stake event
    EType As String ' event type (audience)
    EContact As String ' who to contact
    EComments As String ' extra comments
End Type

Public Type SortCal
    sDate As Date   ' date/time of event
    SLoc As Integer ' location in Events array
    SNext As Integer ' location of next event
End Type



Public Function DayOfWeek(wd As Integer) As String
    Dim ret As String
    
    Select Case wd
    Case 1:
        ret = "Sun"
    Case 2:
        ret = "Mon"
    Case 3:
        ret = "Tue"
    Case 4:
        ret = "Wed"
    Case 5:
        ret = "Thu"
    Case 6:
        ret = "Fri"
    Case 7:
        ret = "Sat"
    Case Else:
        ret = "???"
    End Select
    DayOfWeek = ret
    
End Function

Sub create_html(d1 As Date, d2 As Date)
    Dim i
    Dim dt As Date
    Dim m1 As Integer, m2 As Integer    ' beg & end months
    Dim y1 As Integer, y2 As Integer    ' beg & end years
    Dim mt As Integer, yt As Integer    ' current value in loop
    Dim mx As Integer, yx As Integer    ' ending value for loop
    
    ' set output directory is not set yet
    If Len(OutputDir) = 0 Then
        OutputDir = App.Path
    End If
    
    ' get range of months
    m1 = Month(d1)  ' start date
    y1 = Year(d1)
    m2 = Month(d2)  ' end date
    y2 = Year(d2)

    ' generate calendar for range or rest of year
    mt = m1 ' start at beg month
    yt = y1 ' and beg year
    If y1 < y2 Then
        ' range spans years - do rest of year
        mx = 12
        yx = y1
    Else
        ' range within year - set last month
        mx = m2
        yx = y1
    End If
    ' now generate HTML for this range
    For mt = m1 To mx
        Call create_html_calendar(mt, y1)
    Next
    
End Sub
Sub create_html_calendar(m1 As Integer, y1 As Integer)
    Dim s As String
    
    ' build two digit month number
    If m1 < 10 Then
        s = "0" & m1
    Else
        s = m1
    End If
    
    CalFileName = CalFilePrefix & y1 & "_" & s & ".html"
    EventFileName = "e_" & CalFileName
    Open OutputDir & "\" & CalFileName For Output As #HtmlFile
    Open OutputDir & "\" & EventFileName For Output As #EventFile
    Call create_html_header(m1, y1)
    Call create_html_month(m1, y1)
    Call create_html_footer(m1, y1)
    Close #HtmlFile
    Close #EventFile
End Sub
Sub create_html_header(m1 As Integer, y1 As Integer)
    Dim mstr As String
    Dim Buf As String
    Dim i As Integer
    Dim s As String
    
    mstr = get_month_name_long(m1)
    
    ' creating two files - calendar and events
    Print #HtmlFile, "<html>" & vbCrLf & "<head>" & vbCrLf
    Print #HtmlFile, "<title>" & CalTitle & "-" & mstr & " " & y1 & "</title>"
    ' start event file
    Print #EventFile, "<html>" & vbCrLf & "<head>" & vbCrLf
    Print #EventFile, "<title>Details for " & CalTitle & "-" & mstr & ", " & y1 & "</title>"
    
    ' copy style sheet into both files
    Open App.Path & "\style.txt" For Input As #StyleFile
    Do While Not EOF(StyleFile)
        Line Input #StyleFile, Buf
        Print #HtmlFile, Buf
        Print #EventFile, Buf
    Loop
    Close #StyleFile
    
    Print #HtmlFile, "</head>" & vbCrLf & "<body>"
    Print #EventFile, "</head>" & vbCrLf & "<body>"
    
    ' create links for calendar file
    
    Print #HtmlFile, "<table border='0' cellspacing='2' width='100%' align='center'><tr class='calrow'>"
        For i = 1 To 12
        ' build two digit month number
        If i < 10 Then
            s = "0" & i
        Else
            s = i
        End If
        If i <> m1 Then
            Print #HtmlFile, "<td class='mndx' width='8%'>"
            ' not this month - create link to page
            Print #HtmlFile, "<a href='" & CalFilePrefix & y1 & "_" & s & ".html' class='mndx'>";
        Else
            Print #HtmlFile, "<td class='mndx_curr' width='8%'>"
        End If
        Print #HtmlFile, get_month_name(i);
        If i <> m1 Then
            Print #HtmlFile, "</a>";
        End If
        Print #HtmlFile, "</td>"
    Next
    Print #HtmlFile, "</tr></table>"
    
    ' create start of calendar
    Print #HtmlFile, "<table class='caltab' border='1' cellspacing='0' width='100%'>"
    Print #HtmlFile, "<tr class='calrow'><th colspan='7' class='mname'>" & mstr & " " & y1 & "</th></tr>"
    Print #HtmlFile, "<tr class='calrow'><th class='whdr'>Sunday</th><th class='whdr'>Monday</th><th class='whdr'>Tuesday</th><th class='whdr'>Wednesday</th><th class='whdr'>Thursday</th><th class='whdr'>Friday</th><th class='whdr'>Saturday</th></tr>"
        
    ' print start of event list
    Print #EventFile, "<h1>Details for " & CalTitle & " - " & mstr & ", " & y1 & "</h1>"
    Print #EventFile, "<dl>" & vbCrLf
    
End Sub
Sub create_html_footer(m1 As Integer, y1 As Integer)

    ' create footer for calendar file
    Print #HtmlFile, "<tr class='calrow'><th class='notes' colspan='7'>For Church Use Only</th></tr>"
    Print #HtmlFile, "</table>"
    Print #HtmlFile, "</body>" & vbCrLf & "</html>"
    
    ' create footer for event list
    Print #EventFile, "</dl>" & vbCrLf
    Print #EventFile, "<p class='notes'>For Church Use Only</p>"
    Print #EventFile, "</body>" & vbCrLf & "<html>"
    
End Sub
Sub create_html_month(m1 As Integer, y1 As Integer)
    Dim i As Integer, sun As Integer, j As Integer
    Dim dt As Date
    Dim dmax As Integer   ' number of days in month
    
    ' determine what day the first falls on
    dt = m1 & "/1/" & y1
    j = Weekday(dt)
    
    If j > 1 Then
        ' month starts in middle of week
        Print #HtmlFile, "<tr class='calrow'>"
        For i = 1 To j - 1
            Print #HtmlFile, "<td class='calempty'>&nbsp;</td>"
        Next
    End If
    
    ' determine what days fall on Sunday
    sun = (9 - j) Mod 7
    
    dmax = max_days_in_month(m1, y1)
    
    ' now loop thru month, add events for each day
    For i = 1 To dmax
        If (i Mod 7) = sun Then
            ' new week
            Print #HtmlFile, "</tr><tr class='calrow'>"
        End If
        Print #HtmlFile, "<td class='calcell' valign='top'><table border='0' width='100%' cellspacing='0' cellpadding='2'>"
        Print #HtmlFile, "<tr class='calrow'><td class='calday' width='15'>" & i & "</td><td>&nbsp;</td></tr>"
        Call insert_html_events(m1, i, y1)
        Print #HtmlFile, "</td></tr></table>"
        Print #HtmlFile, "</td>"
    Next
    
    ' finish up last week
    dt = m1 & "/" & dmax & "/" & y1
    j = Weekday(dt)

    If j < 7 Then
        For i = j + 1 To 7
            Print #HtmlFile, "<td class='calempty'>&nbsp;</td>"
        Next
    End If
End Sub
Sub insert_html_events(m1 As Integer, d1 As Integer, y1 As Integer)
    Dim dt As Date, dcurr As Date
    Dim i As Integer
    Dim none As Boolean
    Dim s As String, t As String
    
    dcurr = m1 & "/" & d1 & "/" & y1
    none = True
    For i = 0 To NumEvents - 1
        dt = Events(i).EDate
        If dt = dcurr Then
            t = WriteEventList(i)  ' add to event list
            ' insert this event
            If Events(i).ETime <> "" Then
                s = Format(Events(i).ETime, "hh:mm AM/PM") & " " & Events(i).EDesc
            Else
                s = Events(i).EDesc
            End If
            Print #HtmlFile, "<tr class='calrow'><td class='calevent' colspan='2'><a href='" & EventFileName & "#" & Events(i).EID & "' alt='" & t & "' title='" & t & "' class='calevent' target='det' > " & vbCrLf; ""
            Print #HtmlFile, s & "</a></td></tr>"
            none = False
        End If
    Next
    If none Then
        Print #HtmlFile, "<tr class='calrow'><td>&nbsp;</tr></td>"
    Else
        Print #EventFile, "<hr />"
    End If
End Sub
Function WriteEventList(loc As Integer) As String
    Dim s As String
    Dim h1 As Integer, m1 As Integer
    Dim h2 As Integer, m2 As Integer
    Dim dt1 As Date, dt2 As Date
    Dim ret As String

    ' print whatever time(s) we have
    dt1 = Events(loc).EBegTime
    h1 = Hour(dt1)
    m1 = Minute(dt1)
    If h1 <> 0 And (m1 = 0 Or m1 = 30) Then
        dt2 = Events(loc).EEndTime
        h2 = Hour(dt2)
        m2 = Minute(dt2)
        If h2 <> 0 And (m2 = 0 Or m2 = 30) Then
            ' start and stop times given
            s = h1 & ":" & m1 & "-" & h2 & ":" & m2
            s = Format(dt1, "hh:mm AM/PM") & " to " & Format(dt2, "hh:mm AM/PM")
        Else
            s = h1 & ":" & m1
            s = Format(dt1, "hh:mm AM/PM")
        End If
    ElseIf Events(loc).ETime <> "" Then
        s = Events(loc).ETime
    Else
        s = ""
    End If

    Print #EventFile, "<dt><a name='" & Events(loc).EID & "'>" & Format(Events(loc).EDate, "dddd, mmmm d, yyyy") & " <span class='event_time'>" & s & "</span></a></dt>"
    ret = s
    
    ' sponsoring group
    Select Case Events(loc).EStake
    Case "Y"
        s = "Stake " & Events(loc).EType & " Event"
    Case ""
        s = Events(loc).EType & " Event"
    Case Else
        s = Events(loc).EStake & " " & Events(loc).EType & " Event"
    End Select
    Print #EventFile, "<dd class='event_type'>(" & s & ")</dd>"
    ret = ret & " (" & s & ")"

    Print #EventFile, "<dd class='event_desc'>" & Events(loc).EDesc & "</dd>"
    ' contact info, if present
    If Events(loc).EContact <> "" Then
        ret = ret & ", Contact: " & Events(loc).EContact
        Print #EventFile, "<dd class='event_contact'><b>Contact: </b>" & Events(loc).EContact & "</dd>"
    End If
    ' include any comments
    If Events(loc).EComments <> "" Then
        ret = ret & " [" & Events(loc).EComments & "]"
        Print #EventFile, "<dd class='event_comments'><b>Comments: </b>" & Events(loc).EComments & "</dd>"
    End If
    
    WriteEventList = ret
End Function

Function get_month_name(m As Integer)
    Dim ret As String
    
    Select Case m
    Case 1:
        ret = "Jan"
    Case 2:
        ret = "Feb"
    Case 3:
        ret = "Mar"
    Case 4:
        ret = "Apr"
    Case 5:
        ret = "May"
    Case 6:
        ret = "Jun"
    Case 7:
        ret = "Jul"
    Case 8:
        ret = "Aug"
    Case 9:
        ret = "Sep"
    Case 10:
        ret = "Oct"
    Case 11:
        ret = "Nov"
    Case 12:
        ret = "Dec"
    Case Else
        ret = "???"
    End Select
    
    get_month_name = ret
End Function
Function get_month_name_long(m As Integer)
    Dim ret As String
    
    Select Case m
    Case 1:
        ret = "January"
    Case 2:
        ret = "February"
    Case 3:
        ret = "March"
    Case 4:
        ret = "April"
    Case 5:
        ret = "May"
    Case 6:
        ret = "June"
    Case 7:
        ret = "July"
    Case 8:
        ret = "August"
    Case 9:
        ret = "September"
    Case 10:
        ret = "October"
    Case 11:
        ret = "November"
    Case 12:
        ret = "December"
    Case Else
        ret = "???"
    End Select
    
    get_month_name_long = ret
End Function


Function max_days_in_month(m1 As Integer, y1 As Integer) As Integer
    Dim ret As Integer
    
    Select Case m1
    Case 1:
        ret = 31
    Case 2:
        ' more complicated - check for leap year
        If leap_year(y1) Then
            ret = 29
        Else
            ret = 28
        End If
    Case 3:
        ret = 31
    Case 4:
        ret = 30
    Case 5:
        ret = 31
    Case 6:
        ret = 30
    Case 7:
        ret = 31
    Case 8:
        ret = 31
    Case 9:
        ret = 30
    Case 10:
        ret = 31
    Case 11:
        ret = 30
    Case 12:
        ret = 31
    Case Else:
        ret = 31
    End Select
    
    max_days_in_month = ret
End Function
Function leap_year(y1 As Integer) As Boolean
    Dim ret As Boolean
    
    If (y1 Mod 4) <> 0 Then
        ret = False
    ElseIf (y1 Mod 100) = 0 Then
        ret = False
    Else
        ret = True
    End If
End Function
Sub write_cif_header(filenum As Integer)
    Print #filenum, "Calendar Maker v2.5"
    Print #filenum, CalTitle
    Print #filenum, vbCrLf & "[Birthday]"
    Print #filenum, vbCrLf & "[Anniversary]"
    Print #filenum, vbCrLf & "[Holiday]"
    Print #filenum, vbCrLf & "[Floating Holiday]"

    Print #filenum, vbCrLf & "[Other]"
End Sub

Sub write_cif_record(i As Integer, filenum As Integer)
    Dim dt As Date
    Dim h As Integer
    Dim m As Integer
    Dim s As String
    
        ' if time given, add as string before description
        If Events(i).ETime <> "" Then
            dt = Events(i).ETime
            h = Hour(dt)
            If h > 12 Then h = h - 12
            m = Minute(dt)
            If m = 0 Then
                s = ""
            ElseIf m < 10 Then
                s = ":0" & m
            Else
                s = ":" & m
            End If
            s = h & s & " "
        ElseIf Not IsNull(Events(i).EBegTime) Then
            dt = Events(i).EBegTime
            h = Hour(dt)
            If h > 12 Then h = h - 12
            m = Minute(dt)
            If m = 0 Then
                s = ""
            ElseIf m < 10 Then
                s = ":0" & m
            Else
                s = ":" & m
            End If
            If h = 0 And m = 0 Then
                s = ""
            Else
                s = h & s & " "
            End If
        Else
            s = ""
        End If
        dt = Events(i).EDate    ' get date
        If Len(s) > 0 Then
            Print #filenum, dt & " """ & s & Events(i).EDesc & """"
        Else
            Print #1, dt & " """ & Events(i).EDesc & """"
        End If

End Sub
Sub write_cif_footer(filenum As Integer)
    Print #filenum, vbCrLf & "[End of Data]"
End Sub

Sub sort_calendar()

End Sub

Function BuildConnectionString(dbname As String) As String
    BuildConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;" _
        & "Data Source=" & dbname & ";"
End Function
Function DataBaseOpen() As Boolean
    DataBaseOpen = (DatabaseName <> "")
End Function
Function BuildEventWhere() As String
    Dim ret As String
    Dim i As Integer
    
    For i = 0 To NumEventTypes - 1
        If i = 0 Then
            ret = "'" & EventTypes(i) & "'"
        Else
            ret = ret & ",'" & EventTypes(i) & "'"
        End If
    Next
    
    If NumEventTypes > 0 Then
        ret = "EventType in (" & ret & ")"
    End If
    
    BuildEventWhere = ret
End Function
Sub create_vcal(d1 As Date, d2 As Date)
    Dim i As Long
    Dim dt As Date
    Dim s As String
    
    ' open calendar file
    s = VCalDir & "/" & VCalFileName
    Open s For Output As #VCalFile
    
    ' generate header file
    Call create_vcal_header
    
    For i = 0 To NumEvents - 1
        dt = Events(i).EDate
        If dt >= d1 And dt <= d2 Then
            Call insert_vcal_event(i)
        End If
    Next
    
    ' generate end of file
    Call create_vcal_footer
    
    Close #VCalFile
    
End Sub
Sub create_vcal_header()
    Print #VCalFile, "BEGIN:VCALENDAR"
    Print #VCalFile, "VERSION:1.0"
    Print #VCalFile, "PRODID: PalmDesktop Generated"
End Sub
Sub create_vcal_footer()
    Print #VCalFile, "END:VCALENDAR"
End Sub
Sub insert_vcal_event(ndx As Long)
    
    Print #VCalFile, "BEGIN:VEVENT"
    Print #VCalFile, "SUMMARY:" & Events(ndx).EDesc
    ' build start date/time
    Dim yr As Integer
    Dim mon As Integer
    Dim hr As Integer
    Dim mn As Integer
    Dim s As String
    
    s = format_vcal_date(Events(ndx).EDate, Events(ndx).EBegTime)
    Print #VCalFile, "DTSTART:" & s
    s = format_vcal_date(Events(ndx).EDate, Events(ndx).EEndTime)
    Print #VCalFile, "DTEND:" & s
    Print #VCalFile, "END:VEVENT"
End Sub
Function format_vcal_date(dt As Date, ti As Date) As String
    Dim ret As String
    Dim yr As String
    Dim mon As String
    Dim day As String
    Dim hr As String
    Dim mn As String
    
    yr = DatePart("yyyy", dt)
    mon = DatePart("m", dt)
    If Len(mon) = 1 Then mon = "0" & mon
    day = DatePart("d", dt)
    If Len(day) = 1 Then day = "0" & day
    
    ' get parts of time
    hr = DatePart("h", ti)
    If Len(hr) = 1 Then hr = "0" & hr
    mn = DatePart("n", ti)
    If Len(mn) = 1 Then mn = "0" & mn
    
    ret = yr & mon & day & "T" & hr & mn & "00"
    format_vcal_date = ret
End Function
Function BuildUnitWhere() As String
    Dim ret As String
    Dim i As Integer
    
    If NumUnitNames = 0 Then
        Exit Function
    End If
    
    For i = 0 To NumUnitNames - 1
        If i = 0 Then
            ret = UnitNames(i)
        Else
            ret = ret & "," & UnitNames(i)
        End If
    Next
    
    BuildUnitWhere = "Stake in (" & ret & ")"
End Function
